#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#include "x7170adfutil.h"
#include <lexmark-H/scannerdevice.h>
#include "../lexmark/saneportmonitor.h"
#include "../lexmark/scanerrorcommunicator.h"

#define SCANNER_IOCTL_VENDOR _IOR('U', 0x20, int)
#define SCANNER_IOCTL_PRODUCT _IOR('U', 0x21, int)

SanePortMonitor *sanepm;
ScanErrorCommunicator *ei;
ScannerDevice *sd;
int vendorID = 0x00, prodID = 0x00;
char port[20];
int filedesc = -1;
PortMonitor::PM_Error pmerr;
unsigned char statusADF;
ScannerDevice::SD_ErrCode sderr;
bool paperJam = false;
bool paperOnADF = false;

int openport(void)
{
	// initialize port monitor
  	sanepm = new SanePortMonitor;
  	if (!sanepm)
    	return 1;
	// find the usb device file where scanner is connected
 	for (int index=0; index<=15; index++)
 	{
    	sprintf(port,"/dev/usb/scanner%d",index);
    	filedesc = open(port, O_WRONLY);
    	if (filedesc >= 0)
   		{
      		if (!(ioctl(filedesc, SCANNER_IOCTL_VENDOR, &vendorID)) && !(ioctl(filedesc, SCANNER_IOCTL_PRODUCT, &prodID)))
      		{
        		if ((vendorID == 0x043d) && (prodID == 0x0098))
        		{
          			close(filedesc);
          			break;
        		}
        		close(filedesc);
      		}
      		else
      		{
        		perror("ioctl");
				close(filedesc);      
				return 1;
      		}	
    	}
  	}
  	// set the device file
  	pmerr = sanepm->PM_SetDeviceName(port);
  	if (pmerr != PortMonitor::PM_ERR_SUCCESS)
  	{ 
    	delete sanepm;
    	return 1;
  	}
  
  	// open the port
  	pmerr = sanepm->PM_Open();
  	if (pmerr != PortMonitor::PM_ERR_SUCCESS)
  	{  
    	delete sanepm;
    	return 1;
  	}
  	ei = new ScanErrorCommunicator;
  	if (!ei)
  	{
    	delete sanepm;
    	return -1;
  	}
  	sd = new ScannerDevice(sanepm, ei, "X7170");
	
	return 0;	
}

void closeport(void)
{

}

int getADFStatus(void)
{
	unsigned char adf;
	
  	if (!sd)
  	{
    	delete sanepm;
    	delete ei;
    	return 1;
  	} 

	sderr = sd->SD_GetADFStatus(&adf);
  
   	// close port
  	pmerr = sanepm->PM_Close();
  	if (pmerr != PortMonitor::PM_ERR_SUCCESS)
  	{  
    	delete sanepm;
    	return 1;
  	}
  delete sanepm;

  if( sderr == ScannerDevice::SD_DEVICE_OFF)
  {
	  	printf("Device is off. Make sure the device is powered on.\n");
		return 1;
  }
  else if (sderr != ScannerDevice::SD_OK)
		return 1;


  if(adf == 0x00)
  {
 	  paperOnADF = false;
	  paperJam = false;
  }
  else if(adf == 0x01)
  {
 	paperOnADF = true;
	paperJam = false;
  }
  else if(adf == 0x0F)
  {
 	paperOnADF = true;
	paperJam = true;
  }
  else  //unknown response, reset to false
  {
    paperOnADF = false;
    paperJam = false;
  }

  
  return 0;
}


int x7170adfutil(int numPages, char * filename)
{
	char cmd[255];
	int ret;
	int index=1;
	
	ret = system("which scanadf >/dev/null 2>&1");
	if(ret)
	{
		printf("scanadf frontend not installed. Please install the frontend first!\n");
		return 1;
	}
	if(openport())
	{
		printf("Unable to communicate with scanner.\n"
          "Be sure the device is connected to the computer and powered on.\n");
		return 1;
	}
	if(getADFStatus())
		return 1;
	if(!paperOnADF)
	{
		printf("No paper on ADF. Load paper on ADF and scan again.\n");
		return 1;
	}
	else if(paperJam)
	{
		printf("Paper jam on ADF.  Clear paper jam on ADF and scan again.\n");
		return 1;
	}
	
	sprintf(cmd, "rm -rf /var/tmp/lexmark");
	system(cmd);
	sprintf(cmd, "mkdir /var/tmp/lexmark");
	system(cmd);
	
	for(index=1; index <= numPages; index++)
	{
		if(openport())
		{
			printf("Unable to communicate with scanner.\n"
          		"Be sure the device is connected to the computer and powered on.\n");
			return 1;
		}
		if(getADFStatus())
			return 1;
			
		if(!paperOnADF)
		{
			printf("No more paper on ADF\n");
			break;
		}
		else if(paperJam)
		{
			printf("Paper jam on ADF.  Clear paper jam on ADF and scan again.\n");
			return 1;
		}
	
		sprintf(cmd, "scanadf -e 1 -o  /var/tmp/lexmark/%s01 >/dev/null 2>&1", filename);
		ret = system(cmd);
		if(ret)
		{
			printf("Failed to scan the document. Exiting..\n");
			sprintf(cmd, "rm -rf /var/tmp/lexmark");
			system(cmd);			
			return 1;
		}
		
		sprintf(cmd, "pnmtops  -nocenter -imagewidth 8.5 -imageheight 14 -width 8.5 -height 14 /var/tmp/lexmark/%s01 > /var/tmp/lexmark/%s0%d.ps", filename, filename, index);
		system(cmd);
		if(index > 1)
		{
			//combine the previous image with the currently scanned image into one file
			sprintf(cmd, "cat /var/tmp/lexmark/%s0%d.ps /var/tmp/lexmark/%s0%d.ps > /var/tmp/lexmark/%s", filename, index-1, filename, index, filename);
			system(cmd);
			//delete the files combined
			sprintf(cmd, "rm -rf /var/tmp/lexmark/%s0%d.ps /var/tmp/lexmark/%s0%d.ps", filename, index-1, filename, index);
			system(cmd);
			//rename the combined file to the filename of the last scan
			sprintf(cmd, "mv /var/tmp/lexmark/%s /var/tmp/lexmark/%s0%d.ps", filename, filename, index);
			system(cmd);
		}
	}
	//convert postscript to pdf
	sprintf(cmd, "ps2pdf13 /var/tmp/lexmark/%s0%d.ps %s.pdf >/dev/null 2>&1", filename, index-1, filename);
	ret = system(cmd);
	//display the pdf file in kghostview if installed
	ret = system("which kghostview >/dev/null 2>&1");
	if(ret)
	{
		//printf("No kghostview installed, using xpdf.\n");
		ret = system("which xpdf >/dev/null 2>&1");
		if(ret)
		{
			printf("No xpdf installed, using ggv.\n");
			ret = system("which ggv >/dev/null 2>&1");
			if(ret)
				printf("No pdf viewer installed. Please install application [kghostview, xpdf or ggv] to view file!\n");
			else
			{
				sprintf(cmd, "ggv %s.pdf & >/dev/null 2>&1", filename);
				system(cmd);	
			}
		}
		else
		{
			sprintf(cmd, "xpdf %s.pdf & >/dev/null 2>&1", filename);
			system(cmd);
		}			
	}
	else
	{
    	sprintf(cmd, "kghostview %s.pdf & >/dev/null 2>&1", filename);
		system(cmd);
	}
	//cleanup	
	//sprintf(cmd, "rm -rf %s*.ppm %s*.ps  %s01 out*", filename, filename, filename); 
	sprintf(cmd, "rm -rf /var/tmp/lexmark"); 
	system(cmd);
	printf("Scanned %s.pdf\n", filename);	
	return 0;
}

int main(int argc, char **argv)
{
	int ret;
	//open the port first
	
	if( argc > 4)
	{				
		printf("Wrong number of arguments!\n");
		printf("usage: 	x7170adfutil no_of_pages filename [or]\n");
		printf("x7170adfutil no_of_pages\n");
		return 0;
	}
	else if(argc == 3)
	{
		for(int i=0; i < strlen(argv[1]); i++)
		{
			if(isalpha(argv[1][i])) 
			{
				printf("Invalid number of pages!\n");
				return 0;
			}
			else continue;
		}
		
		if(strtol(argv[1], NULL, 10) > 99)
		{
			printf("Exceeded maximum number of scan pages in ADF.\n");
			return 0;
		}
		else if(strtol(argv[1], NULL, 10) <= 0)
		{
			printf("Invalid number of pages!\n");
			return 0;
		}

		ret = x7170adfutil(strtol(argv[1], NULL, 10), argv[2]);		
	}
	else if(argc == 2)
	{
		for(int i=0; i < strlen(argv[1]); i++)
		{
			if(isalpha(argv[1][i])) 
			{
				printf("Invalid number of pages!\n");
				return 0;
			}
			else continue;
		}
		
		if(strtol(argv[1], NULL, 10) > 99)
		{
			printf("Exceeded maximum number of scan pages in ADF.\n");
			return 0;
		}
		else if(strtol(argv[1], NULL, 10) <= 0)
		{
			printf("Invalid number of pages!\n");
			return 0;
		}

		ret = x7170adfutil(strtol(argv[1], NULL, 10), "out"); //use default filename
	}
	else
		x7170adfutil(99, "out"); //use default filename and scan until there are no more paper on ADF
	
	return ret;
}
