/**************************************************************************
   lexmark.h - SANE library for Lexmark scanners.
   Copyright (C) 2003-2005 Lexmark International, Inc.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.
   **************************************************************************/
#ifndef LEXMARK_H
#define LEXMARK_H

typedef enum
{
  LXK_OPT_NUM_OPTS = 0,
  LXK_OPT_CHANNELS,
  LXK_OPT_HALFTONE_MODE,
  LXK_OPT_SCAN_TYPE,
  LXK_OPT_PREVIEW,
  LXK_OPT_DPI,	
  LXK_OPT_BUTTON_LABEL1,  
  LXK_OPT_BUTTON_LABEL2,
  LXK_OPT_SCANAREA_GROUP,
  LXK_OPT_TL_X,
  LXK_OPT_TL_Y,
  LXK_OPT_BR_X,
  LXK_OPT_BR_Y,
  LXK_OPT_SCAN_SOURCE,
/* must come last: */
  LXK_NUM_OPTIONS
}
Lexmark_Options;

/** @name Option_Value union
 * convenience union to access option values given to the backend
 * @{
 */
#ifndef SANE_OPTION
#define SANE_OPTION 1
typedef union
{
  SANE_Bool b;
  SANE_Word w;
  SANE_Word *wa;		/* word array */
  SANE_String s;
}
Option_Value;
#endif
/* @} */

typedef struct Lexmark_Device
{
  struct Lexmark_Device *next;
  
  SANE_Device sane;
  SANE_Option_Descriptor opt[LXK_NUM_OPTIONS];
  Option_Value val[LXK_NUM_OPTIONS];
  SANE_Parameters params;
  SANE_Int state;
}
Lexmark_Device;

typedef	enum //maps for ScanErrorInterface::ScannerError
{
	LXK_NONE,			/**< Operation results in No errors. */
	LXK_COMMUNICATION_ERROR,	/**< Scanner gets communication errors. This may be a result of
					  an offline condition or the connection to the scanner is
					  broken. */
	LXK_DEVICE_WARMING_UP, 		 /**< Scanner is warming up.(For CCD scanners only) */
	LXK_SCANNER_NOT_LOCKED,		 /**< Device is busy. Scanner can't be locked. */
	LXK_DEVICE_OFF,  		 /**< Device is off */
	LXK_COPIER_NOT_LOCKED,		 /**< Device is busy. Copier can't be locked. */
	LXK_SCAN_STOPPED,		/**< Scanning was stopped from device(AIO) */
	LXK_MEMORY_OUT,		/**< An out of memory error occured */
	LXK_UNEXPECTED_ERROR		/**< An unexpected error occured */
}
Lexmark_Errors;

#endif /* LEXMARK_H */
