/**************************************************************************
   saneportmonitor.h - SANE library for Lexmark scanners.
                       - class declaration of SanePortMonitor class
   Copyright (C) 2003-2005 Lexmark International, Inc.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.
   **************************************************************************/
#ifndef SANE_PORT_MONITOR_H
#define SANE_PORT_MONITOR_H

#define SANE_PM_MAX_DEVICE_NAME 255
#define SANE_PM_INVALID_HANDLE -1
#define SANE_PM_MAX_PACKET_SIZE 4096

#include <lexmark-H/portmonitor.h>
#include "../include/sane/sane.h"

class SanePortMonitor: public PortMonitor
{

	public:
     
		SanePortMonitor();
    		virtual ~SanePortMonitor();
		
    		PM_Error PM_Open();
    		PM_Error PM_Close();
    		long PM_Write(
			unsigned char * buffer,
			unsigned long size);
    		long PM_Read(
			unsigned char * buffer,
			unsigned long size);

    		PM_Error PM_RegisterEventCallback(
			PM_EventCallback pfnCallback);
    		PM_Error PM_UnregisterEventCallback();

    		PM_Error PM_WaitForData(
			long timeout);

		PM_Error PM_SetDeviceName(
			SANE_String_Const devname);
		PM_Error PM_AttachDevice(
			const char * name,
			SANE_Status (* attach) (const char * dev));	
		
	private:
		
    		SANE_Int m_PortHandle;
    		SANE_String m_DeviceName;
};

#endif // SANE_PORT_MONITOR_H
