/**************************************************************************
   scanerrorcommunicator.cpp - SANE library for Lexmark scanners.
                              - implements dispatch mechanism for reporting errors
   Copyright (C) 2003-2005 Lexmark International, Inc.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.
   **************************************************************************/
#include "scanerrorcommunicator.h"
#include <stdio.h>
#include <string.h>
#include <time.h>

ScanErrorCommunicator::ScanErrorCommunicator() 
{
}

ScanErrorCommunicator::~ScanErrorCommunicator()
{
}

void ScanErrorCommunicator::ReportError(ScanErrorInterface::ScannerError error)
{
  char errStr[200];
  FILE * scanerror_log;
  time_t t = time(NULL);
  
  if (error == ScanErrorInterface::ERR_NONE)
   return;
  else if (error == ScanErrorInterface::ERR_COMMUNICATION_ERROR)
    sprintf(errStr, "ERROR: %s   Unable to communicate with scanner.\n\
          Be sure the device is connected to the computer and powered on.\n", ctime(&t));
  else if (error == ScanErrorInterface::ERR_DEVICE_WARMING_UP)
    sprintf(errStr, "INFO: %s   Device is warming up.\n\
          Please wait while the scanner lamp is warming.\n", ctime(&t));
  else if (error == ScanErrorInterface::ERR_SCANNER_NOT_LOCKED)
    sprintf(errStr, "ERROR: %s   Scanner can't be locked. Device is busy\n", ctime(&t));
  else if (error == ScanErrorInterface::ERR_DEVICE_OFF)
    sprintf(errStr, "ERROR: %s   Device is off. Make sure the device is powered on.\n", ctime(&t));
  else if (error == ScanErrorInterface::ERR_COPIER_NOT_LOCKED)
    sprintf(errStr, "ERROR: %s   Copier can't be locked. Device is busy\n", ctime(&t));
  else if (error == ScanErrorInterface::ERR_SCAN_STOPPED)
    sprintf(errStr, "INFO: %s   Scan was stopped from device.\n", ctime(&t));
  else if (error == ScanErrorInterface::ERR_MEMORY_OUT)
    sprintf(errStr, "ERROR: %s   Scanner data exceeded maximum, aborting..\n", ctime(&t));
  else if (error == ScanErrorInterface::ERR_ADF_PAPER_JAM)
    sprintf(errStr, "ERROR: %s   Paper jam on Automatic Document Feeder.  Please remove the paper and clear the paper jam error and scan again\n", ctime(&t));
  else /* Unexpected Error */
    sprintf(errStr, "ERROR: %s   Scanner encountered unexpected error. Try these solutions:\n\
          - Make sure the All-In-One is powered on.\
          - Disconnect and reconnect the USB cable.\
          - Disconnect and reconnect the All-In-One's power cable.\
          - Restart your computer.\n", ctime(&t));

  scanerror_log = fopen("/var/log/scanerror_log", "a");
  if (scanerror_log != NULL)
  {
	fputs(errStr, scanerror_log);
  	fclose(scanerror_log);
  }
  return;
}

ScanErrorInterface::UserDecision ScanErrorCommunicator::GetUserDecision()
{
  return ScanErrorInterface::CONTINUE;
}
