#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <lexmark-H/scannerdevice.h>
#include "../lexmark/saneportmonitor.h"
#include "../lexmark/scanerrorcommunicator.h"

#define SCANNER_IOCTL_VENDOR _IOR('U', 0x20, int)
#define SCANNER_IOCTL_PRODUCT _IOR('U', 0x21, int)

int main(int argc, char **argv)
{
  int filedesc = -1;
  int vendorID = 0x00, prodID = 0x00;
  char port[20];
  unsigned char buffer[]="The GIMP,";
  ScannerDevice::SD_ErrCode sderr;
  PortMonitor::PM_Error pmerr;

  SanePortMonitor *sanepm;
  ScanErrorCommunicator *ei;
  ScannerDevice *sd;
  
  // initialize port monitor
  sanepm = new SanePortMonitor;
  if (!sanepm)
    return -1;

  // find the usb device file where scanner is connected
  for (int index=0; index<=15; index++)
  {
    sprintf(port,"/dev/usb/scanner%d",index);
    filedesc = open(port, O_WRONLY);
    if (filedesc >= 0)
    {
      if (!(ioctl(filedesc, SCANNER_IOCTL_VENDOR, &vendorID)) && !(ioctl(filedesc, SCANNER_IOCTL_PRODUCT, &prodID)))
      {
        if ((vendorID == 0x043d) && (prodID == 0x0098))
        {
          close(filedesc);
          break;
        }
        close(filedesc);
      }
      else
      {
        perror("ioctl");
	close(filedesc);      
	return -1;
      }
    }
  }
  
  // set the device file
  pmerr = sanepm->PM_SetDeviceName(port);
  if (pmerr != PortMonitor::PM_ERR_SUCCESS)
  {  
    delete sanepm;
    return -1;
  }
  
  // open the port
  pmerr = sanepm->PM_Open();
  if (pmerr != PortMonitor::PM_ERR_SUCCESS)
  {  
    delete sanepm;
    return -1;
  }

  ei = new ScanErrorCommunicator;
  if (!ei)
  {
    delete sanepm;
    return -1;
  }
  
  sd = new ScannerDevice(sanepm, ei, "X7170");
  if (!sd)
  {
    delete sanepm;
    delete ei;
    return -1;
  }
  
  // set scan applications to be displayed in LCD
  sderr = sd->SD_SetScanApps(buffer);
  delete sd;
  delete ei;
  
  // close port
  pmerr = sanepm->PM_Close();
  if (pmerr != PortMonitor::PM_ERR_SUCCESS)
  {  
    delete sanepm;
    return -1;
  }
  delete sanepm;

  if (sderr != ScannerDevice::SD_OK)
  {
    printf("setting of p6250 scan applications failed!\n");
    return -1;
  }
  
  return 0;
}

